from datetime import datetime
from typing import List, Optional

import arrow
from cyclonedx.model import XsUri
from cyclonedx.model.component import Component
from cyclonedx.model.vulnerability import (
    BomTarget,
    Vulnerability,
    VulnerabilityRating,
    VulnerabilityReference,
    VulnerabilitySource,
)

from .cve_search import get_cve_search
from .exceptions import CVESearchQueryException, OSVQueryException
from .osv import get_osv
from .osv.schemas import Vuln


def to_datetime(s: str) -> datetime:
    arrw = arrow.get(s)
    return arrw.datetime


class VulnerabilityFactory:
    @staticmethod
    def from_osv_vuln(vuln: Vuln) -> Vulnerability:
        references: List[VulnerabilityReference] = []
        for ref in vuln.references:
            source: Optional[VulnerabilitySource] = None
            if ref.url is not None:
                source = VulnerabilitySource(url=XsUri(ref.url))

            if source is not None:
                references.append(VulnerabilityReference(source=source))

        rating: Optional[VulnerabilityRating] = None
        if vuln.cve_id is not None:
            try:
                cve_search = get_cve_search()
                res = cve_search.query(vuln.cve_id)
                rating = VulnerabilityRating(score=res.cvss, severity=res.severity)
            except CVESearchQueryException:
                pass

        vulnerability = Vulnerability(
            id=vuln.id,
            detail=vuln.details,
            updated=to_datetime(vuln.modified),
            published=to_datetime(vuln.published),
        )
        if rating is not None:
            vulnerability.ratings = [rating]

        return vulnerability

    @staticmethod
    def from_component(component: Component) -> List[Vulnerability]:
        from loguru import logger

        logger.info(component.name)
        logger.info(component.version)
        if component.version is None:
            return []

        vulnerabilities: List[Vulnerability] = []

        try:
            osv = get_osv()
            res = osv.query(name=component.name, version=component.version)
        except OSVQueryException:
            return vulnerabilities

        for vuln in res.vulns:
            vulnerability = VulnerabilityFactory.from_osv_vuln(vuln)
            vulnerability.affects = [BomTarget(ref=component.bom_ref.value)]
            vulnerabilities.append(vulnerability)

        logger.info(vulnerabilities)

        return vulnerabilities
